from dataclasses import dataclass, field
from typing import Any, Dict, List, Optional

from netspresso.trainer.models.base import ArchitectureConfig, CheckpointConfig, ModelConfig


@dataclass
class ResNet18ArchitectureConfig(ArchitectureConfig):
    backbone: Dict[str, Any] = field(
        default_factory=lambda: {
            "name": "resnet",
            "params": {
                "block_type": "basicblock",
                "norm_type": "batch_norm",
                "return_stage_idx": None,
                "split_stem_conv": False,
                "first_stage_shortcut_conv": False,
            },
            "stage_params": [
                {
                    "channels": 64,
                    "num_blocks": 2,
                },
                {
                    "channels": 128,
                    "num_blocks": 2,
                    "replace_stride_with_dilation": False,
                    "replace_stride_with_pooling": False,
                },
                {
                    "channels": 256,
                    "num_blocks": 2,
                    "replace_stride_with_dilation": False,
                    "replace_stride_with_pooling": False,
                },
                {
                    "channels": 512,
                    "num_blocks": 2,
                    "replace_stride_with_dilation": False,
                    "replace_stride_with_pooling": False,
                },
            ],
        }
    )


@dataclass
class ResNet34ArchitectureConfig(ArchitectureConfig):
    backbone: Dict[str, Any] = field(
        default_factory=lambda: {
            "name": "resnet",
            "params": {
                "block_type": "basicblock",
                "norm_type": "batch_norm",
                "return_stage_idx": None,
                "split_stem_conv": False,
                "first_stage_shortcut_conv": False,
            },
            "stage_params": [
                {
                    "channels": 64,
                    "num_blocks": 3,
                },
                {
                    "channels": 128,
                    "num_blocks": 4,
                    "replace_stride_with_dilation": False,
                    "replace_stride_with_pooling": False,
                },
                {
                    "channels": 256,
                    "num_blocks": 6,
                    "replace_stride_with_dilation": False,
                    "replace_stride_with_pooling": False,
                },
                {
                    "channels": 512,
                    "num_blocks": 3,
                    "replace_stride_with_dilation": False,
                    "replace_stride_with_pooling": False,
                },
            ],
        }
    )


@dataclass
class ResNet50ArchitectureConfig(ArchitectureConfig):
    backbone: Dict[str, Any] = field(
        default_factory=lambda: {
            "name": "resnet",
            "params": {
                "block_type": "bottleneck",
                "norm_type": "batch_norm",
                "return_stage_idx": None,
                "split_stem_conv": False,
                "first_stage_shortcut_conv": False,
            },
            "stage_params": [
                {
                    "channels": 64,
                    "num_blocks": 3,
                },
                {
                    "channels": 128,
                    "num_blocks": 4,
                    "replace_stride_with_dilation": False,
                    "replace_stride_with_pooling": False,
                },
                {
                    "channels": 256,
                    "num_blocks": 6,
                    "replace_stride_with_dilation": False,
                    "replace_stride_with_pooling": False,
                },
                {
                    "channels": 512,
                    "num_blocks": 3,
                    "replace_stride_with_dilation": False,
                    "replace_stride_with_pooling": False,
                },
            ],
        }
    )


@dataclass
class ClassificationResNet18ModelConfig(ModelConfig):
    task: str = "classification"
    name: str = "resnet18"
    architecture: ArchitectureConfig = field(
        default_factory=lambda: ResNet18ArchitectureConfig(
            head={
                "name": "fc",
                "params": {
                    "num_layers": 1,
                    "intermediate_channels": None,
                    "act_type": None,
                    "dropout_prob": 0.0,
                },
            }
        )
    )
    postprocessor: Optional[Dict[str, Any]] = None
    losses: List[Dict[str, Any]] = field(
        default_factory=lambda: [{"criterion": "cross_entropy", "label_smoothing": 0.1, "weight": None}]
    )


@dataclass
class ClassificationResNet34ModelConfig(ModelConfig):
    task: str = "classification"
    name: str = "resnet34"
    architecture: ArchitectureConfig = field(
        default_factory=lambda: ResNet34ArchitectureConfig(
            head={
                "name": "fc",
                "params": {
                    "num_layers": 1,
                    "intermediate_channels": None,
                    "act_type": None,
                    "dropout_prob": 0.0,
                },
            }
        )
    )
    postprocessor: Optional[Dict[str, Any]] = None
    losses: List[Dict[str, Any]] = field(
        default_factory=lambda: [{"criterion": "cross_entropy", "label_smoothing": 0.1, "weight": None}]
    )


@dataclass
class ClassificationResNet50ModelConfig(ModelConfig):
    task: str = "classification"
    name: str = "resnet50"
    architecture: ArchitectureConfig = field(
        default_factory=lambda: ResNet50ArchitectureConfig(
            head={
                "name": "fc",
                "params": {
                    "num_layers": 1,
                    "intermediate_channels": None,
                    "act_type": None,
                    "dropout_prob": 0.0,
                },
            }
        )
    )
    postprocessor: Optional[Dict[str, Any]] = None
    losses: List[Dict[str, Any]] = field(
        default_factory=lambda: [{"criterion": "cross_entropy", "label_smoothing": 0.1, "weight": None}]
    )


@dataclass
class SegmentationResNet50ModelConfig(ModelConfig):
    task: str = "segmentation"
    name: str = "resnet50"
    architecture: ArchitectureConfig = field(
        default_factory=lambda: ResNet50ArchitectureConfig(
            backbone={
                "name": "resnet",
                "params": {
                    "block_type": "bottleneck",
                    "norm_type": "batch_norm",
                    "return_stage_idx": [0, 1, 2, 3],
                    "split_stem_conv": False,
                    "first_stage_shortcut_conv": False,
                },
                "stage_params": [
                    {
                        "channels": 64,
                        "num_blocks": 3,
                    },
                    {
                        "channels": 128,
                        "num_blocks": 4,
                        "replace_stride_with_dilation": False,
                        "replace_stride_with_pooling": False,
                    },
                    {
                        "channels": 256,
                        "num_blocks": 6,
                        "replace_stride_with_dilation": False,
                        "replace_stride_with_pooling": False,
                    },
                    {
                        "channels": 512,
                        "num_blocks": 3,
                        "replace_stride_with_dilation": False,
                        "replace_stride_with_pooling": False,
                    },
                ],
            },
            head={
                "name": "all_mlp_decoder",
                "params": {
                    "intermediate_channels": 256,
                    "classifier_dropout_prob": 0.0,
                },
            },
        )
    )
    postprocessor: Optional[Dict[str, Any]] = None
    losses: List[Dict[str, Any]] = field(
        default_factory=lambda: [{"criterion": "seg_cross_entropy", "ignore_index": 255, "weight": None}]
    )


@dataclass
class DetectionResNet50ModelConfig(ModelConfig):
    task: str = "detection"
    name: str = "resnet50"
    architecture: ArchitectureConfig = field(
        default_factory=lambda: ResNet50ArchitectureConfig(
            backbone={
                "name": "resnet",
                "params": {
                    "block_type": "bottleneck",
                    "norm_type": "batch_norm",
                    "return_stage_idx": [0, 1, 2, 3],
                    "split_stem_conv": False,
                    "first_stage_shortcut_conv": False,
                },
                "stage_params": [
                    {
                        "channels": 64,
                        "num_blocks": 3,
                    },
                    {
                        "channels": 128,
                        "num_blocks": 4,
                        "replace_stride_with_dilation": False,
                        "replace_stride_with_pooling": False,
                    },
                    {
                        "channels": 256,
                        "num_blocks": 6,
                        "replace_stride_with_dilation": False,
                        "replace_stride_with_pooling": False,
                    },
                    {
                        "channels": 512,
                        "num_blocks": 3,
                        "replace_stride_with_dilation": False,
                        "replace_stride_with_pooling": False,
                    },
                ],
            },
            neck={
                "name": "fpn",
                "params": {
                    "num_outs": 4,
                    "start_level": 0,
                    "end_level": -1,
                    "add_extra_convs": False,
                    "relu_before_extra_convs": False,
                },
            },
            head={
                "name": "anchor_decoupled_head",
                "params": {
                    # Anchor parameters
                    "anchor_sizes": [
                        [
                            32,
                        ],
                        [
                            64,
                        ],
                        [
                            128,
                        ],
                        [
                            256,
                        ],
                    ],
                    "aspect_ratios": [0.5, 1.0, 2.0],
                    "num_layers": 1,
                    "norm_type": "batch_norm",
                },
            },
        )
    )
    postprocessor: Optional[Dict[str, Any]] = field(
        default_factory=lambda: {
            "params": {
                # postprocessor - decode
                "topk_candidates": 1000,
                "score_thresh": 0.05,
                # postprocessor - nms
                "nms_thresh": 0.45,
                "class_agnostic": False,
            },
        }
    )
    losses: List[Dict[str, Any]] = field(
        default_factory=lambda: [
            {"criterion": "retinanet_loss", "weight": None},
        ]
    )



from .schedulers import (
    CosineAnnealingLRWithCustomWarmUp,
    CosineAnnealingWarmRestartsWithCustomWarmUp,
    MultiStepLR,
    PolynomialLRWithWarmUp,
    StepLR,
)

__all__ = [
    "StepLR",
    "PolynomialLRWithWarmUp",
    "CosineAnnealingLRWithCustomWarmUp",
    "CosineAnnealingWarmRestartsWithCustomWarmUp",
    "MultiStepLR",
]



from dataclasses import dataclass, field
from typing import Any, Dict, List, Optional

from netspresso.trainer.models.base import ArchitectureConfig, ModelConfig


@dataclass
class PIDNetArchitectureConfig(ArchitectureConfig):
    full: Dict[str, Any] = field(
        default_factory=lambda: {
            "name": "pidnet",
            "m": 2,
            "n": 3,
            "channels": 32,
            "ppm_channels": 96,
            "head_channels": 128,
        }
    )


@dataclass
class PIDNetModelConfig(ModelConfig):
    task: str = "segmentation"
    name: str = "pidnet_s"
    architecture: ArchitectureConfig = field(default_factory=lambda: PIDNetArchitectureConfig())
    postprocessor: Optional[Dict[str, Any]] = None
    losses: List[Dict[str, Any]] = field(
        default_factory=lambda: [
            {"criterion": "pidnet_loss", "ignore_index": 255, "weight": None},
        ]
    )



from netspresso.exceptions.common import AdditionalData, PyNPException


class NotValidChannelAxisRangeException(PyNPException):
    def __init__(self, reshape_channel_axis: int):
        message = f"The reshape_channel_axis value is in the range [0, 1, -1, -2], but got {reshape_channel_axis}"
        super().__init__(
            data=AdditionalData(origin="pynp"),
            error_code="",
            name=self.__class__.__name__,
            message=message,
        )


class EmptyCompressionParamsException(PyNPException):
    def __init__(self):
        message = "The available_layer.values all empty. please put in the available_layer.values to compress."
        super().__init__(
            data=AdditionalData(origin="pynp"),
            error_code="",
            name=self.__class__.__name__,
            message=message,
        )


class NotValidSlampRatioException(PyNPException):
    def __init__(self, ratio: float):
        message = (f"The ratio range for SLAMP is 0 < ratio < 1, but got `{ratio}`.",)
        super().__init__(
            data=AdditionalData(origin="pynp"),
            error_code="",
            name=self.__class__.__name__,
            message=message,
        )


class NotValidVbmfRatioException(PyNPException):
    def __init__(self, ratio: float):
        message = f"The ratio range for VBMF is -1 <= ratio <= 1, but got `{ratio}`."
        super().__init__(
            data=AdditionalData(origin="pynp"),
            error_code="",
            name=self.__class__.__name__,
            message=message,
        )


class NotFillInputLayersException(PyNPException):
    def __init__(self):
        message = "Please fill in Input Layers fields."
        super().__init__(
            data=AdditionalData(origin="pynp"),
            error_code="",
            name=self.__class__.__name__,
            message=message,
        )


class FailedUploadModelException(PyNPException):
    def __init__(self):
        message = "Failed to upload model."
        super().__init__(
            data=AdditionalData(origin="pynp"),
            error_code="",
            name=self.__class__.__name__,
            message=message,
        )



from dataclasses import asdict, dataclass, field
from typing import Any, Dict, List


@dataclass
class BaseOptimizer:
    no_bias_decay: bool = False
    no_norm_weight_decay: bool = False
    overwrite: Any = None

    def asdict(self) -> Dict:
        return asdict(self)


@dataclass
class Adadelta(BaseOptimizer):
    name: str = "adadelta"
    lr: float = 1.0
    rho: float = 0.9
    weight_decay: float = 0.0


@dataclass
class Adagrad(BaseOptimizer):
    name: str = "adagrad"
    lr: float = 1e-2
    lr_decay: float = 0.0
    weight_decay: float = 0.0


@dataclass
class Adam(BaseOptimizer):
    name: str = "adam"
    lr: float = 1e-3
    betas: List[float] = field(default_factory=lambda: [0.9, 0.999])
    weight_decay: float = 0.0


@dataclass
class Adamax(BaseOptimizer):
    name: str = "adamax"
    lr: float = 2e-3
    betas: List[float] = field(default_factory=lambda: [0.9, 0.999])
    weight_decay: float = 0.0


@dataclass
class AdamW(BaseOptimizer):
    name: str = "adamw"
    lr: float = 1e-3
    betas: List[float] = field(default_factory=lambda: [0.9, 0.999])
    weight_decay: float = 0.01


@dataclass
class RMSprop(BaseOptimizer):
    name: str = "rmsprop"
    lr: float = 1e-2
    alpha: float = 0.99
    momentum: float = 0.0
    weight_decay: float = 0.0
    eps: float = 1e-8


@dataclass
class SGD(BaseOptimizer):
    name: str = "sgd"
    lr: float = 1e-2
    momentum: float = 0.0
    weight_decay: float = 0.0
    nesterov: bool = False



from pathlib import Path
from typing import Dict, List, Optional
from urllib import request

from loguru import logger

from netspresso.analytics import netspresso_analytics
from netspresso.base import NetsPressoBase
from netspresso.clients.auth import TokenHandler
from netspresso.clients.compressor import compressor_client_v2
from netspresso.clients.compressor.v2.schemas import (
    ModelBase,
    Options,
    RecommendationOptions,
    RequestAutomaticCompressionParams,
    RequestAvailableLayers,
    RequestCreateCompression,
    RequestCreateModel,
    RequestCreateRecommendation,
    RequestUpdateCompression,
    RequestUploadModel,
    RequestValidateModel,
    ResponseCompression,
    ResponseSelectMethod,
    UploadFile,
)
from netspresso.clients.launcher import launcher_client_v2
from netspresso.compressor.utils.onnx import export_onnx
from netspresso.enums import CompressionMethod, Framework, RecommendationMethod, ServiceTask, Status
from netspresso.exceptions.compressor import FailedUploadModelException
from netspresso.metadata.compressor import CompressorMetadata
from netspresso.utils import FileHandler
from netspresso.utils.metadata import MetadataHandler


class CompressorV2(NetsPressoBase):
    def __init__(self, token_handler: TokenHandler) -> None:
        """Initialize the Compressor."""

        super().__init__(token_handler)

    def _update_metadata_for_trainer(self, metadata: CompressorMetadata, input_model_path: str):
        if (Path(input_model_path).parent / "metadata.json").exists():
            trained_data = FileHandler.load_json(Path(input_model_path).parent / "metadata.json")
            metadata.update_model_info_for_trainer(
                task=trained_data["model_info"]["task"],
                model=trained_data["model_info"]["model"],
                dataset=trained_data["model_info"]["dataset"],
            )
            metadata.update_training_info(
                epochs=trained_data["training_info"]["epochs"],
                batch_size=trained_data["training_info"]["batch_size"],
                learning_rate=trained_data["training_info"]["learning_rate"],
                optimizer=trained_data["training_info"]["optimizer"],
            )
            metadata.update_is_retrainable(is_retrainable=True)
            metadata.update_training_result(training_result=trained_data["training_result"])

        return metadata

    def initialize_metadata(self, output_dir, input_model_path, compression_method, ratio, framework, input_shapes):
        def create_metadata_with_status(status, error_message=None):
            metadata = CompressorMetadata()
            metadata.status = status
            if error_message:
                logger.error(error_message)
            return metadata

        try:
            metadata = CompressorMetadata()
        except Exception as e:
            error_message = f"An unexpected error occurred during metadata initialization: {e}"
            metadata = create_metadata_with_status(Status.ERROR, error_message)
        except KeyboardInterrupt:
            warning_message = "Compression task was interrupted by the user."
            metadata = create_metadata_with_status(Status.STOPPED, warning_message)
        finally:
            metadata.input_model_path = Path(input_model_path).resolve().as_posix()
            metadata.compression_info.method = compression_method
            metadata.compression_info.ratio = ratio
            metadata.model_info.framework = framework
            metadata.model_info.input_shapes = input_shapes
            metadata = self._update_metadata_for_trainer(metadata, input_model_path)
            MetadataHandler.save_metadata(data=metadata, folder_path=output_dir)

        return metadata

    def _get_available_options(self, compressed_model_info, default_model_path: str):
        if compressed_model_info.detail.framework in [Framework.PYTORCH, Framework.ONNX]:
            export_onnx(default_model_path, compressed_model_info.detail.input_layers)
            options_response = launcher_client_v2.converter.read_framework_options(
                access_token=self.token_handler.tokens.access_token,
                framework=Framework.ONNX,
            )
        else:
            options_response = launcher_client_v2.converter.read_framework_options(
                access_token=self.token_handler.tokens.access_token,
                framework=Framework.TENSORFLOW_KERAS,
            )

        available_options = options_response.data

        # TODO: Will be removed when we support DLC in the future
        available_options = [
            available_option for available_option in available_options if available_option.framework != "dlc"
        ]

        return available_options

    def _postprocess_metadata(
        self,
        metadata: CompressorMetadata,
        model_info: ModelBase,
        compression_info: ResponseCompression,
        default_model_path: str,
        extension: str,
    ):
        compressed_model_info = self.get_model(model_id=compression_info.input_model_id)
        available_options = self._get_available_options(compressed_model_info, default_model_path)

        if compressed_model_info.detail.framework in [Framework.PYTORCH, Framework.ONNX]:
            metadata.update_compressed_onnx_model_path(default_model_path.with_suffix(".onnx").as_posix())
        metadata.compression_info.layers = compression_info.available_layers
        metadata.compression_info.options = compression_info.options
        metadata.update_compressed_model_path(default_model_path.with_suffix(extension).as_posix())
        metadata.update_results(model=model_info, compressed_model=compressed_model_info)
        metadata.update_status(status=Status.COMPLETED)
        metadata.update_available_options(available_options)

        return metadata

    def finalize_compression_process(
        self,
        metadata: CompressorMetadata,
        model_info: ModelBase,
        compression_info: ResponseCompression,
        output_dir: str,
    ):
        default_model_path = FileHandler.get_default_model_path(folder_path=output_dir)
        extension = FileHandler.get_extension(framework=model_info.detail.framework)
        self.download_model(compression_info.input_model_id, default_model_path.with_suffix(extension))
        metadata = self._postprocess_metadata(metadata, model_info, compression_info, default_model_path, extension)

        return metadata

    def upload_model(
        self,
        input_model_path: str,
        input_shapes: List[Dict[str, int]] = None,
        framework: Framework = Framework.PYTORCH,
    ) -> ModelBase:
        """Upload a model for compression.

        Args:
            input_model_path (str): The file path where the model is located.
            input_shapes (List[Dict[str, int]], optional): Input shapes of the model. Defaults to [].
            framework (Framework): The framework of the model.

        Raises:
            e: If an error occurs while uploading the model.

        Returns:
            ModelBase: Uploaded model object.
        """

        self.token_handler.validate_token()

        try:
            logger.info("Uploading Model...")

            FileHandler.check_input_model_path(input_model_path)

            object_name = Path(input_model_path).name

            create_model_request = RequestCreateModel(object_name=object_name)
            create_model_response = compressor_client_v2.create_model(
                request_data=create_model_request,
                access_token=self.token_handler.tokens.access_token,
                verify_ssl=self.token_handler.verify_ssl,
            )

            file_content = FileHandler.read_file_bytes(file_path=input_model_path)
            upload_model_request = RequestUploadModel(url=create_model_response.data.presigned_url)
            file = UploadFile(file_name=object_name, file_content=file_content)
            upload_model_response = compressor_client_v2.upload_model(
                request_data=upload_model_request,
                file=file,
                access_token=self.token_handler.tokens.access_token,
                verify_ssl=self.token_handler.verify_ssl,
            )

            if not upload_model_response:
                # TODO: Confirm upload success
                raise FailedUploadModelException()

            validate_model_request = RequestValidateModel(framework=framework, input_layers=input_shapes)
            validate_model_response = compressor_client_v2.validate_model(
                ai_model_id=create_model_response.data.ai_model_id,
                request_data=validate_model_request,
                access_token=self.token_handler.tokens.access_token,
                verify_ssl=self.token_handler.verify_ssl,
            )

            model_info = validate_model_response.data

            logger.info(f"Upload model successfully. Model ID: {model_info.ai_model_id}")

            return model_info

        except Exception as e:
            logger.error(f"Upload model failed. Error: {e}")
            raise e

    def get_model(self, model_id: str) -> ModelBase:
        self.token_handler.validate_token()

        try:
            logger.info("Getting model...")
            read_model_response = compressor_client_v2.read_model(
                ai_model_id=model_id,
                access_token=self.token_handler.tokens.access_token,
                verify_ssl=self.token_handler.verify_ssl,
            )
            model_info = read_model_response.data

            logger.info("Get model successfully.")

            return model_info

        except Exception as e:
            logger.error(f"Get model failed. Error: {e}")
            raise e

    def download_model(self, model_id: str, local_path: str) -> None:
        self.token_handler.validate_token()

        try:
            logger.info("Downloading model...")
            download_link = compressor_client_v2.download_model(
                ai_model_id=model_id,
                access_token=self.token_handler.tokens.access_token,
                verify_ssl=self.token_handler.verify_ssl,
            )
            request.urlretrieve(download_link.data.presigned_url, local_path)
            logger.info(f"Model downloaded at {Path(local_path)}")

        except Exception as e:
            logger.error(f"Download model failed. Error: {e}")
            raise e

    def select_compression_method(
        self,
        model_id: str,
        compression_method: CompressionMethod,
        options: Optional[Options] = Options(),
    ) -> ResponseSelectMethod:
        """Select a compression method for a model.

        Args:
            model_id (str): The ID of the model.
            compression_method (CompressionMethod): The selected compression method.
            options(Options, optional): The options for pruning method.

        Raises:
            e: If an error occurs while selecting the compression method.

        Returns:
            ResponseSelectMethod: The compression information for the selected compression method.
        """

        self.token_handler.validate_token()

        try:
            logger.info("Selecting compression method...")

            get_available_layers_request = RequestAvailableLayers(
                compression_method=compression_method,
                options=options,
            )
            get_available_layers_response = compressor_client_v2.get_available_layers(
                ai_model_id=model_id,
                request_data=get_available_layers_request,
                access_token=self.token_handler.tokens.access_token,
                verify_ssl=self.token_handler.verify_ssl,
            )

            available_layers_info = get_available_layers_response.data

            logger.info("Select compression method successfully.")

            return available_layers_info

        except Exception as e:
            logger.error(f"Select compression method failed. Error: {e}")
            raise e

    def get_compression(self, compression_id: str) -> ResponseCompression:
        self.token_handler.validate_token()

        try:
            logger.info("Getting compression...")
            read_compression_response = compressor_client_v2.read_compression(
                compression_id=compression_id,
                access_token=self.token_handler.tokens.access_token,
                verify_ssl=self.token_handler.verify_ssl,
            )

            compression_info = read_compression_response.data

            logger.info("Get compression successfully.")

            return compression_info

        except Exception as e:
            logger.error(f"Get compression failed. Error: {e}")
            raise e

    def upload_dataset(self, compression_id: str, dataset_path: str) -> None:
        self.token_handler.validate_token()

        try:
            logger.info("Uploading dataset...")
            file_content = FileHandler.read_file_bytes(file_path=dataset_path)
            object_name = Path(dataset_path).name
            file = UploadFile(file_name=object_name, file_content=file_content)
            compressor_client_v2.upload_dataset(
                compression_id=compression_id,
                file=file,
                access_token=self.token_handler.tokens.access_token,
                verify_ssl=self.token_handler.verify_ssl,
            )
            logger.info("Upload dataset successfully.")

        except Exception as e:
            logger.error(f"Upload dataset failed. Error: {e}")
            raise e

    def compress_model(
        self,
        compression: ResponseSelectMethod,
        output_dir: str,
        dataset_path: Optional[str] = None,
    ) -> CompressorMetadata:
        """Compress a model using the provided compression information.

        Args:
            compression (CompressionInfo): The information about the compression.
            output_dir (str): The local path to save the compressed model.
            dataset_path (str, optional): The path of the dataset used for nuclear norm compression method. Default is None.

        Raises:
            e: If an error occurs while compressing the model.

        Returns:
            CompressorMetadata: Compress metadata.
        """

        netspresso_analytics.send_event(
            event_name="manual_compression_using_np",
            event_params={
                "compression_method": compression.compression_method,
            },
        )

        output_dir = FileHandler.create_unique_folder(folder_path=output_dir)
        metadata: CompressorMetadata = self.initialize_metadata(
            output_dir=output_dir,
            input_model_path="",
            compression_method=compression.compression_method,
            ratio="",
            framework="",
            input_shapes=[],
        )

        try:
            logger.info("Compressing model...")
            if metadata.status in [Status.ERROR, Status.STOPPED]:
                return metadata

            self.validate_token_and_check_credit(service_task=ServiceTask.ADVANCED_COMPRESSION)

            create_compression_request = RequestCreateCompression(
                ai_model_id=compression.input_model_id,
                compression_method=compression.compression_method,
                options=compression.options,
            )
            create_compression_response = compressor_client_v2.create_compression(
                request_data=create_compression_request,
                access_token=self.token_handler.tokens.access_token,
                verify_ssl=self.token_handler.verify_ssl,
            )

            for available_layers in compression.available_layers:
                if available_layers.values:
                    available_layers.use = True

            if dataset_path and compression.compression_method in [CompressionMethod.PR_NN, CompressionMethod.PR_SNP]:
                self.upload_dataset(create_compression_response.data.compression_id, dataset_path)

            update_compression_request = RequestUpdateCompression(
                available_layers=compression.available_layers,
                options=compression.options,
            )
            update_compression_response = compressor_client_v2.compress_model(
                compression_id=create_compression_response.data.compression_id,
                request_data=update_compression_request,
                access_token=self.token_handler.tokens.access_token,
                verify_ssl=self.token_handler.verify_ssl,
            )
            compression_info = update_compression_response.data
            model_info = self.get_model(model_id=compression.input_model_id)
            metadata.update_model_info(model_info.detail.framework, model_info.detail.input_layers)
            metadata = self.finalize_compression_process(metadata, model_info, compression_info, output_dir)

            self.print_remaining_credit(service_task=ServiceTask.ADVANCED_COMPRESSION)

            logger.info(f"Compress model successfully. Compressed Model ID: {compression_info.input_model_id}")

        except Exception as e:
            metadata = self.handle_error(metadata, ServiceTask.ADVANCED_COMPRESSION, e.args[0])
        except KeyboardInterrupt:
            metadata = self.handle_stop(metadata, ServiceTask.ADVANCED_COMPRESSION)
        finally:
            MetadataHandler.save_metadata(data=metadata, folder_path=output_dir)

    def recommendation_compression(
        self,
        compression_method: CompressionMethod,
        recommendation_method: RecommendationMethod,
        recommendation_ratio: float,
        input_model_path: str,
        output_dir: str,
        input_shapes: List[Dict[str, int]],
        framework: Framework = Framework.PYTORCH,
        options: RecommendationOptions = RecommendationOptions(),
        dataset_path: Optional[str] = None,
    ) -> CompressorMetadata:
        """Compress a recommendation-based model using the given compression and recommendation methods.

        Args:
            compression_method (CompressionMethod): The selected compression method.
            recommendation_method (RecommendationMethod): The selected recommendation method.
            recommendation_ratio (float): The compression ratio recommended by the recommendation method.
            input_model_path (str): The file path where the model is located.
            output_dir (str): The local path to save the compressed model.
            input_shapes (List[Dict[str, int]]): Input shapes of the model.
            framework (Framework, optional): The framework of the model.
            options(Options, optional): The options for pruning method.
            dataset_path (str, optional): The path of the dataset used for nuclear norm compression method. Default is None.

        Raises:
            e: If an error occurs while performing recommendation compression.

        Returns:
            CompressorMetadata: Compress metadata.
        """

        netspresso_analytics.send_event(
            event_name="recommendation_compression_using_np",
            event_params={
                "framework": framework,
                "compression_method": compression_method,
                "recommendation_method": recommendation_method,
                "recommendation_ratio": recommendation_ratio,
            },
        )

        output_dir = FileHandler.create_unique_folder(folder_path=output_dir)
        metadata = self.initialize_metadata(
            output_dir=output_dir,
            input_model_path=input_model_path,
            compression_method=compression_method,
            ratio=recommendation_ratio,
            framework=framework,
            input_shapes=input_shapes,
        )

        try:
            logger.info("Compressing recommendation-based model...")
            if metadata.status in [Status.ERROR, Status.STOPPED]:
                return metadata

            self.validate_token_and_check_credit(service_task=ServiceTask.ADVANCED_COMPRESSION)

            model_info = self.upload_model(input_model_path, input_shapes, framework)

            create_compression_request = RequestCreateCompression(
                ai_model_id=model_info.ai_model_id,
                compression_method=compression_method,
                options=options,
            )
            create_compression_response = compressor_client_v2.create_compression(
                request_data=create_compression_request,
                access_token=self.token_handler.tokens.access_token,
                verify_ssl=self.token_handler.verify_ssl,
            )

            if dataset_path and compression_method in [CompressionMethod.PR_NN, CompressionMethod.PR_SNP]:
                self.upload_dataset(create_compression_response.data.compression_id, dataset_path)

            logger.info("Calculating recommendation values...")
            create_recommendation_request = RequestCreateRecommendation(
                recommendation_method=recommendation_method,
                recommendation_ratio=recommendation_ratio,
                options=options,
            )
            create_recommendation_response = compressor_client_v2.create_recommendation(
                compression_id=create_compression_response.data.compression_id,
                request_data=create_recommendation_request,
                access_token=self.token_handler.tokens.access_token,
                verify_ssl=self.token_handler.verify_ssl,
            )

            logger.info("Compressing model...")
            update_compression_request = RequestUpdateCompression(
                available_layers=create_recommendation_response.data.available_layers,
                options=options,
            )
            update_compression_response = compressor_client_v2.compress_model(
                compression_id=create_compression_response.data.compression_id,
                request_data=update_compression_request,
                access_token=self.token_handler.tokens.access_token,
                verify_ssl=self.token_handler.verify_ssl,
            )
            compression_info = update_compression_response.data
            metadata = self.finalize_compression_process(metadata, model_info, compression_info, output_dir)

            self.print_remaining_credit(service_task=ServiceTask.ADVANCED_COMPRESSION)

            logger.info(
                f"Recommendation compression successfully. Compressed Model ID: {compression_info.input_model_id}"
            )

        except Exception as e:
            metadata = self.handle_error(metadata, ServiceTask.ADVANCED_COMPRESSION, e.args[0])
        except KeyboardInterrupt:
            metadata = self.handle_stop(metadata, ServiceTask.ADVANCED_COMPRESSION)
        finally:
            MetadataHandler.save_metadata(data=metadata, folder_path=output_dir)

        return metadata

    def automatic_compression(
        self,
        input_model_path: str,
        output_dir: str,
        input_shapes: List[Dict[str, int]],
        framework: Framework = Framework.PYTORCH,
        compression_ratio: float = 0.5,
    ) -> CompressorMetadata:
        """Compress a model automatically based on the given compression ratio.

        Args:
            input_model_path (str): The file path where the model is located.
            output_dir (str): The local path to save the compressed model.
            input_shapes (List[Dict[str, int]]): Input shapes of the model.
            framework (Framework, optional): The framework of the model.
            compression_ratio (float, optional): The compression ratio for automatic compression. Defaults to 0.5.

        Raises:
            e: If an error occurs while performing automatic compression.

        Returns:
            CompressorMetadata: Compress metadata.
        """
        netspresso_analytics.send_event(
            event_name="automatic_compression_using_np",
            event_params={
                "framework": framework,
                "compression_ratio": compression_ratio,
            },
        )

        output_dir = FileHandler.create_unique_folder(folder_path=output_dir)
        metadata = self.initialize_metadata(
            output_dir=output_dir,
            input_model_path=input_model_path,
            compression_method=CompressionMethod.PR_L2,
            ratio=compression_ratio,
            framework=framework,
            input_shapes=input_shapes,
        )

        try:
            logger.info("Compressing automatic-based model...")
            if metadata.status in [Status.ERROR, Status.STOPPED]:
                return metadata

            self.validate_token_and_check_credit(service_task=ServiceTask.AUTOMATIC_COMPRESSION)

            model_info = self.upload_model(input_model_path, input_shapes, framework)

            logger.info("Compressing model...")
            automatic_compression_request = RequestAutomaticCompressionParams(compression_ratio=compression_ratio)
            automatic_compression_response = compressor_client_v2.compress_model_with_automatic(
                ai_model_id=model_info.ai_model_id,
                request_data=automatic_compression_request,
                access_token=self.token_handler.tokens.access_token,
                verify_ssl=self.token_handler.verify_ssl,
            )
            compression_info = automatic_compression_response.data
            metadata = self.finalize_compression_process(metadata, model_info, compression_info, output_dir)

            self.print_remaining_credit(service_task=ServiceTask.AUTOMATIC_COMPRESSION)

            logger.info(f"Automatic compression successfully. Compressed Model ID: {compression_info.input_model_id}")

        except Exception as e:
            metadata = self.handle_error(metadata, ServiceTask.AUTOMATIC_COMPRESSION, e.args[0])
        except KeyboardInterrupt:
            metadata = self.handle_stop(metadata, ServiceTask.AUTOMATIC_COMPRESSION)
        finally:
            MetadataHandler.save_metadata(data=metadata, folder_path=output_dir)

        return metadata



from dataclasses import dataclass, field
from typing import Any, Dict, List, Optional

from netspresso.trainer.models.base import ArchitectureConfig, ModelConfig
from netspresso.trainer.models.mobilenetv3 import MobileNetV3SmallArchitectureConfig


@dataclass
class PoseEstimationMobileNetV3SmallModelConfig(ModelConfig):
    task: str = "pose_estimation"
    name: str = "mobilenet_v3_small"
    architecture: ArchitectureConfig = field(
        default_factory=lambda: MobileNetV3SmallArchitectureConfig(
            head={
                "name": "rtmcc",
                "params": {
                    "conv_kernel": 7,
                    "attention_channels": 256,
                    "attention_act_type": "silu",
                    "attention_pos_enc": False,
                    "s": 128,
                    "expansion_factor": 2,
                    "dropout_rate": 0.0,
                    "drop_path": 0.0,
                    "use_rel_bias": False,
                    "target_size": [256, 256],
                    "backbone_stride": 32,
                },
            }
        )
    )
    postprocessor: Optional[Dict[str, Any]] = field(
        default_factory=lambda: {
            "params": {
                "simcc_split_ratio": 2.0,
            },
        }
    )
    losses: List[Dict[str, Any]] = field(default_factory=lambda: [{"criterion": "rtmcc_loss", "weight": None}])



import json
from dataclasses import asdict, dataclass, field
from typing import Any, Dict, List

from netspresso.enums.metadata import Status, TaskType

from .common import TargetDevice


@dataclass
class InputShape:
    batch: int = 1
    channel: int = 3
    dimension: List[int] = field(default_factory=list)


@dataclass
class ModelInfo:
    data_type: str = ""
    framework: str = ""
    input_shapes: List[InputShape] = field(default_factory=lambda: [InputShape()])


@dataclass
class ConvertInfo:
    target_framework: str = ""
    target_device_name: str = ""
    data_type: str = ""
    software_version: str = ""
    model_file_name: str = ""
    convert_task_uuid: str = ""
    input_model_uuid: str = ""
    output_model_uuid: str = ""


@dataclass
class ConverterMetadata:
    status: Status = Status.IN_PROGRESS
    task_type: TaskType = TaskType.CONVERT
    converted_model_path: str = ""
    model_info: ModelInfo = field(default_factory=ModelInfo)
    convert_info: ConvertInfo = field(default_factory=ConvertInfo)
    available_devices: List[Dict] = field(default_factory=list)

    def asdict(self) -> Dict:
        _dict = json.loads(json.dumps(asdict(self)))
        return _dict

    def update_status(self, status: Status) -> None:
        self.status = status

    def update_model_info(self, data_type: str, framework: str, input_shape: Any) -> None:
        self.model_info.data_type = data_type
        self.model_info.framework = framework
        self.model_info.input_shapes = [
            InputShape(
                batch=input_shape.batch,
                channel=input_shape.channel,
                dimension=[int(x.strip()) for x in input_shape.input_size.split(",")],
            )
        ]

    def update_convert_info(
        self,
        target_framework: str,
        target_device_name: str,
        data_type: str,
        software_version: str,
        model_file_name: str,
        convert_task_uuid: str,
        input_model_uuid: str,
        output_model_uuid: str,
    ) -> None:
        self.convert_info.target_framework = target_framework
        self.convert_info.target_device_name = target_device_name
        self.convert_info.data_type = data_type
        self.convert_info.software_version = software_version
        self.convert_info.model_file_name = model_file_name
        self.convert_info.convert_task_uuid = convert_task_uuid
        self.convert_info.input_model_uuid = input_model_uuid
        self.convert_info.output_model_uuid = output_model_uuid

    def update_converted_model_path(self, converted_model_path):
        self.converted_model_path = converted_model_path

    def update_available_devices(self, available_devices: List[Dict]) -> None:
        self.available_devices = [
            TargetDevice(
                display_name=device.display_name,
                display_brand_name=device.display_brand_name,
                device_name=device.device_name,
                software_version=device.software_version,
                software_version_display_name=device.software_version_display_name,
                hardware_type=device.hardware_type,
            )
            for device in available_devices
        ]



from dataclasses import asdict
from enum import Enum

from loguru import logger

from netspresso.clients.launcher.v2.interfaces import TaskInterface
from netspresso.clients.launcher.v2.schemas import (
    AuthorizationHeader,
    RequestConvert,
    ResponseConvertDownloadModelUrlItem,
    ResponseConvertFrameworkOptionItems,
    ResponseConvertOptionItems,
    ResponseConvertStatusItem,
    ResponseConvertTaskItem,
    UploadFile,
)
from netspresso.clients.utils.requester import Requester
from netspresso.enums import LauncherTask


class ConvertTaskAPI(TaskInterface):
    def __init__(self, url):
        self.task_type = LauncherTask.CONVERT.value
        self.base_url = url
        self.task_base_url = f"{self.base_url}/{self.task_type}/tasks"
        self.option_base_url = f"{self.base_url}/{self.task_type}/options"
        self.model_base_url = f"{self.base_url}/{self.task_type}/models"

    @staticmethod
    def custom_asdict_factory(data):
        def convert_value(obj):
            if isinstance(obj, Enum):
                return obj.value
            return obj

        return {k: convert_value(v) for k, v in data}

    def start(
        self,
        request_body: RequestConvert,
        headers: AuthorizationHeader,
        file: UploadFile = None,
    ) -> ResponseConvertTaskItem:
        endpoint = f"{self.task_base_url}"

        logger.info(f"Request_Body: {asdict(request_body)}")
        response = Requester().post_as_form(
            url=endpoint,
            request_body=asdict(request_body, dict_factory=self.custom_asdict_factory),
            headers=headers.to_dict(),
            binary=file.files if file else None,
        )
        return ResponseConvertTaskItem(**response.json())

    def cancel(self, headers: AuthorizationHeader, task_id: str) -> ResponseConvertTaskItem:
        endpoint = f"{self.task_base_url}/{task_id}/cancel"
        response = Requester().post_as_json(url=endpoint, request_body={}, headers=headers.to_dict())
        return ResponseConvertTaskItem(**response.json())

    def read(self, headers: AuthorizationHeader, task_id: str) -> ResponseConvertTaskItem:
        endpoint = f"{self.task_base_url}/{task_id}"
        response = Requester().get(url=endpoint, headers=headers.to_dict())
        return ResponseConvertTaskItem(**response.json())

    def delete(self, headers: AuthorizationHeader, task_id: str) -> ResponseConvertTaskItem:
        endpoint = f"{self.task_base_url}/{task_id}"
        response = Requester().delete(url=endpoint, headers=headers.to_dict())
        return ResponseConvertTaskItem(**response.json())

    def status(self, headers: AuthorizationHeader, task_id: str) -> ResponseConvertStatusItem:
        endpoint = f"{self.task_base_url}/{task_id}"
        response = Requester().get(url=endpoint, headers=headers.to_dict())
        return ResponseConvertStatusItem(**response.json())

    def options(self, headers: AuthorizationHeader) -> ResponseConvertOptionItems:
        endpoint = f"{self.option_base_url}"
        response = Requester().get(url=endpoint, headers=headers.to_dict())
        return ResponseConvertOptionItems(**response.json())

    def options_by_model_framework(
        self, headers: AuthorizationHeader, model_framework: str
    ) -> ResponseConvertFrameworkOptionItems:
        endpoint = f"{self.option_base_url}/frameworks/{model_framework}"
        response = Requester().get(url=endpoint, headers=headers.to_dict())
        return ResponseConvertFrameworkOptionItems(**response.json())

    def option_by_target_framework(
        self, headers: AuthorizationHeader, target_framework: str
    ) -> ResponseConvertOptionItems:
        endpoint = f"{self.option_base_url}/details/{target_framework}"
        response = Requester().get(url=endpoint, headers=headers.to_dict())
        return ResponseConvertOptionItems(**response.json())

    def get_download_url(
        self, headers: AuthorizationHeader, convert_task_uuid: str
    ) -> ResponseConvertDownloadModelUrlItem:
        endpoint = f"{self.model_base_url}/{convert_task_uuid}"
        response = Requester().get(url=endpoint, headers=headers.to_dict())
        return ResponseConvertDownloadModelUrlItem(**response.json())



from netspresso.clients.utils.requester import Requester


class AuthAPI:
    def __init__(self, url: str):
        self.url = url

    def login(self, request_body):
        endpoint = f"{self.url}/login"

        response = Requester.post_as_json(url=endpoint, request_body=request_body)

        return response.json()



from dataclasses import dataclass


@dataclass
class UserAgreementBase:
    privacy_policy_agreement: bool = False
    marketing_agreement: bool = False
    personal_information_agreement: bool = False
    accessing_age_agreement: bool = False
    terms_of_service_agreement: bool = False



from typing import List

from netspresso.exceptions.common import AdditionalData, PyNPException


class NotSupportedSuffixException(PyNPException):
    def __init__(self, available_suffixes: List, suffix: str):
        message = f"The suffix supports {available_suffixes}. The entered suffix is {suffix}."
        super().__init__(
            data=AdditionalData(origin="pynp"),
            error_code="",
            name=self.__class__.__name__,
            message=message,
        )



from pathlib import Path

from setuptools import find_packages, setup

version = (Path("netspresso") / "VERSION").read_text().strip()

long_description = Path("README.md").read_text(encoding="UTF8")

install_requires = Path("requirements.txt").read_text().split('\n')

setup(
    name="netspresso",
    version=version,
    author="NetsPresso",
    author_email="netspresso@nota.ai",
    description="PyNetsPresso",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/Nota-NetsPresso/PyNetsPresso",
    install_requires=install_requires,
    packages=find_packages(exclude=("tests",)),
    package_data={"netspresso.clients": ["configs/*.ini"], "netspresso": ["VERSION"]},
    python_requires=">=3.8",
    classifiers=[
        "Intended Audience :: Developers",
        "Intended Audience :: Science/Research",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Topic :: Scientific/Engineering :: Artificial Intelligence",
    ],
)



from dataclasses import dataclass
from typing import List, Optional

from qai_hub.client import Device

from netspresso.enums.base import StrEnum


class ComputeUnit(StrEnum):
    ALL = "all"
    NPU = "npu"
    GPU = "gpu"
    CPU = "cpu"


@dataclass
class CommonOptions:
    """
    Common options for all tasks.

    Args:
        compute_unit: Specifies the target compute unit(s)

    Note:
        For details, see `CommonOptions in QAI Hub API <https://app.aihub.qualcomm.com/docs/hub/api.html#common-options>`_.
    """

    compute_unit: Optional[List[ComputeUnit]] = None

    def normalize_compute_units(self):
        if self.compute_unit is None:
            return None
        sorted_units = sorted(self.compute_unit)
        return ",".join(sorted_units)


def normalize_device_name(device: Optional[List[Device]] = None):
    if device is None:
        return None

    if not isinstance(device, list):
        return device.name

    sorted_devices = sorted([d.name for d in device])
    return ",".join(sorted_devices)



import numpy as np


def voc_color_map(N=256, normalized=False, brightness_factor=1.5):
    def bitget(byteval, idx):
        return (byteval & (1 << idx)) != 0

    dtype = "float32" if normalized else "uint8"
    cmap = np.zeros((N, 3), dtype=dtype)
    for i in range(N):
        r = g = b = 0
        c = i
        for j in range(8):
            r = r | (bitget(c, 0) << (7 - j))
            g = g | (bitget(c, 1) << (7 - j))
            b = b | (bitget(c, 2) << (7 - j))
            c = c >> 3

        # Adjust brightness
        r = min(255, max(0, r * brightness_factor))
        g = min(255, max(0, g * brightness_factor))
        b = min(255, max(0, b * brightness_factor))

        cmap[i] = np.array([r, g, b])

    if normalized:
        cmap = cmap / 255
    return cmap



from dataclasses import dataclass, field
from pathlib import Path
from typing import List, Optional, Union


@dataclass
class ModelSaveOptions:
    save_optimizer_state: bool = True
    save_best_only: bool = False
    best_model_criterion: str = "loss"  # metric or loss
    sample_input_size: List[int] = field(default_factory=lambda: [512, 512])
    onnx_export_opset: int = 13  # Recommend in range [13, 17]
    validation_epoch: int = 10
    save_checkpoint_epoch: Optional[int] = None

    def __post_init__(self):
        if self.save_checkpoint_epoch is None:
            self.save_checkpoint_epoch = self.validation_epoch


@dataclass
class Metrics:
    classwise_analysis: bool = False
    metric_names: Optional[List[str]] = None


@dataclass
class LoggingConfig:
    project_id: Optional[str] = None
    output_dir: Union[Path, str] = "./outputs"
    tensorboard: bool = True
    mlflow: bool = False
    image: bool = True
    stdout: bool = True
    model_save_options: ModelSaveOptions = field(default_factory=ModelSaveOptions)
    metrics: Metrics = field(default_factory=Metrics)



from .compression import CompressionMethod, GroupPolicy, LayerNorm, Policy, RecommendationMethod, StepOp
from .config import EndPointProperty, EnvironmentType, ServiceModule, ServiceName
from .credit import MembershipType, ServiceCredit, ServiceTask
from .device import DeviceName, DisplaySoftwareVersion, HardwareType, SoftwareVersion, TaskStatus
from .inference import Runtime
from .metadata import Status, TaskType
from .model import DataType, Extension, Framework, OriginFrom
from .module import Module
from .quantize import OnnxOperator, QuantizationMode, QuantizationPrecision, SimilarityMetric
from .tao.action import ConvertAction, ExperimentAction
from .task import LauncherTask, Task, TaskStatusForDisplay
from .train import Optimizer, Scheduler

__all__ = [
    "ServiceCredit",
    "ServiceTask",
    "TaskType",
    "Status",
    "CompressionMethod",
    "RecommendationMethod",
    "Policy",
    "GroupPolicy",
    "LayerNorm",
    "Task",
    "Framework",
    "Framework",
    "Extension",
    "OriginFrom",
    "DataType",
    "DeviceName",
    "SoftwareVersion",
    "DisplaySoftwareVersion",
    "HardwareType",
    "TaskStatus",
    "Module",
    "ConvertAction",
    "ExperimentAction",
    "StepOp",
    "MembershipType",
    "DisplaySoftwareVersion",
    "LauncherTask",
    "TaskStatusForDisplay",
    "EnvironmentType",
    "ServiceModule",
    "EndPointProperty",
    "ServiceName",
    "Optimizer",
    "Scheduler",
    "Runtime",
    "QuantizationPrecision",
    "OnnxOperator",
    "QuantizationMode",
    "SimilarityMetric",
]



from dataclasses import dataclass

from netspresso.enums import LauncherTask


@dataclass
class RequestModelUploadUrl:
    """ """

    object_name: str
    task: LauncherTask = LauncherTask.CONVERT.value


@dataclass
class RequestUploadModel:
    """ """

    url: str


@dataclass
class RequestValidateModel:
    """ """

    ai_model_id: str
    display_name: str



from .common import (
    AuthorizationHeader,
    DeviceInfo,
    ModelOption,
    RequestPagination,
    ResponseItem,
    ResponseItems,
    ResponsePaginationItems,
    TaskInfo,
    TaskOption,
    UploadDataset,
    UploadFile,
)
from .model import (
    InputLayer,
    ModelBase,
    RequestModelUploadUrl,
    RequestUploadModel,
    RequestValidateModel,
    ResponseModelItem,
    ResponseModelItems,
    ResponseModelOptions,
    ResponseModelStatus,
    ResponseModelUploadUrl,
)
from .task import (
    RequestBenchmark,
    RequestConvert,
    RequestQuantizeTask,
    ResponseBenchmarkFrameworkOptionItems,
    ResponseBenchmarkOptionItems,
    ResponseBenchmarkStatusItem,
    ResponseBenchmarkTaskItem,
    ResponseConvertDownloadModelUrlItem,
    ResponseConvertFrameworkOptionItems,
    ResponseConvertOptionItems,
    ResponseConvertStatusItem,
    ResponseConvertTaskItem,
    ResponseQuantizeDownloadModelUrlItem,
    ResponseQuantizeOptionItems,
    ResponseQuantizeStatusItem,
    ResponseQuantizeTaskItem,
)
